/* -*-C++-*- */
/* Copyright (c) 1999 Regents of the University of California.
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


/* ---------------------------------------------------------
 *   NetworkTCP:
 *      Factory Class, returns SecureTcp* which the caller must
 *	manage.
 *   
 * --------------------------------------------------------- */

#include <iostream>
#include "NetworkTcp.h"

using std::cerr;
using std::endl;

#define DEBUG 1
#ifdef DEBUG
# include <errno.h>
#endif

/* ---------------------------------------------------------
 *   NetworkTCP:
 *   
 * --------------------------------------------------------- */

NetworkTCP::NetworkTCP()
: client_Key("key.pem"), client_Cert("nettest.pem"), 
  server_Key("key.pem"), server_Cert("nettest.pem")
{
    sockAccept = new TCPNet();
}

NetworkTCP::~NetworkTCP()
{
    delete sockAccept;
}


/* ---------------------------------------------------------
 *   testCallBackFunction:
 *   
 * --------------------------------------------------------- */

int
NetworkTCP::testCallBackFunction( bool isServer, CallBackPtr pass_cb )
{
    SecureTcp * coordcon;
    (isServer)?
 	coordcon = new SecureTcp( server_Cert, server_Key):
 	coordcon = new SecureTcp( client_Cert, client_Key);

    if( pass_cb ) coordcon->setPassword_cb( *pass_cb );

    if( coordcon->init( false, "" ) == false ){
        cerr << "NetworkTcp connect coordcon->init() failed\n";
	delete coordcon;
	return -1;
    }
    delete coordcon;
    return 0;
}


/* ---------------------------------------------------------
 * newTCP_accept: 
 *      Return a securetcp(Serverside: nettest.pem, key.pem)
 *	after accepting a connection
 *
 *  	void* pass_cb :  callback function for ssl connection
 * 
 * --------------------------------------------------------- */

SecureTcp*  
NetworkTCP::newTCP_accept( CallBackPtr pass_cb )
{
    if( sockAccept->getSock() < 0 ){ // NOT initialized
        if( sockAccept->openSocket(true)  < 0 ){
        	cerr << "NetworkTcp sockAccept->openSocket() failed\n";
        	cerr << "Error message: " << strerror(errno) << endl;
        	return NULL;
        }
        sockAccept->Listen();
    }

    if( sockAccept->Accept() <= 0 ){
        cerr << "NetworkTcp sockAccept->Accept() failed\n";
        cerr << "Error message: " << strerror(errno) << endl;
        return NULL;
    }

    SecureTcp *coordcon = new SecureTcp(server_Cert, server_Key);
    if( pass_cb ) coordcon->setPassword_cb( *pass_cb );

    if( coordcon->init( true, "" ) == false ){
        cerr << "NetworkTcp accept coordcon->init() failed\n";
	delete coordcon;
	return NULL;
    }

    if( coordcon->doAccept(&secInfo, sockAccept) < 0 ){
        cerr << "NetworkTcp coordcon->doAccept failed\n";
	delete coordcon;
        return NULL;
    }

    return coordcon;
}

/* ---------------------------------------------------------
 * newTCP_connect: Return a securetcp after connecting to an accept
 * 
 *  	void* pass_cb : Callback function for ssl connection
 *
 *  	NOTE:2 certificates with same key OK, 2 keys with same cert NOT OK 
 *
 * --------------------------------------------------------- */

SecureTcp*  
NetworkTCP::newTCP_connect( string hostname, CallBackPtr pass_cb )
{
    SecureTcp * coordcon = new SecureTcp( client_Cert, client_Key);
    if( pass_cb ) coordcon->setPassword_cb( *pass_cb );

    if( coordcon->init( false, hostname ) == false ){
        cerr << "NetworkTcp connect coordcon->init() failed\n";
	delete coordcon;
	return NULL;
    }

    if( coordcon->doConnect(&secInfo, hostname ) < 0 ){
        cerr << "NetworkTcp connect coordcon->doConnect() failed\n";
	delete coordcon;
	return NULL;
    }

    return coordcon;
}
