/* -*-C++-*- */
/*
 *  Copyright (c) 2000 Regents of the University of California.
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley 
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

/* ---------------------------------------------------------
 *  Results:   Container for all results objects.
 *
 *    One results object will be instantiated in the lblnettest object, 
 *    as tests are created a new specific results object will be created 
 *    within this class.  This new object will be filled with test data 
 *    by a PingTest(UDPPing) or IperfTest(Perfsocket).  In lblnettest the 
 *    Results object is created, and a pointer to it is passed to  PingTest 
 *    and IperfTest.  Then createNewPingTest() is called to create the 
 *    test object, eg pingTest, to create a recepticle for that test.  
 *    The test will then write its data into that recepticle by passing a 
 *    reference to itself (eg UDPPing &) to Results.
 *
 * --------------------------------------------------------- */

#include "Results.h"
#include "IperfResults.h"
#include "PingResults.h"
#include "PerfSocket.hpp"
#include "PingTestInfo.h"

#if USE_NETLOGGER
#include "logging.h"
#include "NetLogger.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

using std::cerr;
using std::cout;
using std::endl;


/* ---------------------------------------------------------
 *  Results:
 *	netloghost - Full Canonical name of host where
 *		     NetLogger is running
 *
 * --------------------------------------------------------- */

Results::Results( void )
   : MyDebug("Results: "), isNLping(false), isNLiperf(false), isNLSO(false),
     isPrintPing(false), isPrintIperf(false), isPrintSO(true), isNLenabled(false), isReportEndOfTest(true),
     isFinalReport(false)
{
#if USE_NETLOGGER
   Nlh = NULL;
#endif
}

#if USE_NETLOGGER
Results::Results( string netloghost )
   : MyDebug("Results: "), isNLping(false), isNLiperf(false), isNLSO(false),
     isPrintPing(false), isPrintIperf(false), isPrintSO(true), isNLenabled(true), isReportEndOfTest(true),
     isFinalReport(false)
{
   Nlh = NULL;
   setNetLogHost( netloghost );
}
#endif

Results::~Results( void )
{
#if USE_NETLOGGER
   if ( Nlh != NULL )
      NetLoggerClose(Nlh);
#endif
}


/* ---------------------------------------------------------
 *  RefreshNetLoggerHandle()
 *
 * --------------------------------------------------------- */

#if USE_NETLOGGER
void 
Results::RefreshNetLoggerHandle()
{
   string arg1 = "Nettest";
   string arg2("x-netlog://");
   arg2 = arg2 + _netloghost;

   // should have been closed!
   if (Nlh != NULL)
   {
        NetLoggerClose(Nlh);
   }

   // Init handle
   Nlh = NetLoggerOpen(const_cast<char*>(arg1.c_str()),
                       const_cast<char*>(arg2.c_str()), 0);
   if( Nlh == NULL )
   {
        cerr << "Error: couldn't connect to netlogd on '"
             << _netloghost << "'" << endl;
        cerr << "Warning: not producing NetLogger output" << endl;
   }
}
#endif  // USE_NETLOGGER


/* ---------------------------------------------------------
 * setNetLogHost :
 *
 * --------------------------------------------------------- */

#if USE_NETLOGGER
int
Results::setNetLogHost( string netloghost )
{
   _netloghost = netloghost;
   RefreshNetLoggerHandle();
   return 0;
}
#endif


/* ---------------------------------------------------------
 * newPingResults:
 *
 * --------------------------------------------------------- */

void
Results::newPingResults( PingResults pingR )
{
#if USE_NETLOGGER
    if(  isNLenabled && isNLping ){
	RefreshNetLoggerHandle();
        if( Nlh != NULL )
            pingR.netLogResults_ulm(Nlh);
        else
            cerr << "Error: NetLogger handle is NULL\n";
    }
#endif
    if( isPrintPing )
        pingR.printResults();
}


/* ---------------------------------------------------------
 * IperfResults:
 *	Called by PerfSocket
 *
 * --------------------------------------------------------- */

void         
Results::iperfResults( IperfResults& ipR )
{
    IperfResults *ipR_ptr  = new IperfResults( ipR );

    if( isPrintIperf )
        ipR_ptr->printResults();

    if( !isFinalReport )
    {
        printDebug(3, "iperfResults - !isReportEndOfTest\n");
#if USE_NETLOGGER
        if( isNLenabled && isNLiperf ){
	    RefreshNetLoggerHandle();
            if( Nlh != NULL )
                ipR_ptr->netLogResults_ulm(Nlh);
            else
                cerr << "Error: netlogger handle is NULL\n";
        }
#endif
        delete ipR_ptr;
    }
    else{
        printDebug(3, "iperfResults - isReportEndOfTest \nipR_ptr->getGroupID()=%d\n",ipR_ptr->getGroupID());
        ip_Map.insert( std::make_pair(ipR_ptr->getGroupID(),ipR_ptr) );
    }
}


/* ---------------------------------------------------------
 * iperfResults_EndOfTest:  
 *	Called at end of test for multiple thread tests to get 
 *	a report of the total bits/s and bytes.
 *
 *   	ip_Map: Contains final tallies from each threaded test. 
 *	      	No intermediate reports should be kept here.
 *
 * --------------------------------------------------------- */

int
Results::iperfResults_EndOfTest( pthread_t group_id )
{
    IperfResults *ip_ptr = NULL;
    IPMapi   pos;
    double       endTime = 0;
    double       temp    = 0;
    max_size_t   bytesLen = 0;

    printDebug(3, "iperfResults_EndOfTest group_id %d\n", group_id);
    if( (pos=ip_Map.find(group_id)) == ip_Map.end() ){
        printDebug(3, "call to iperfResult_EndOfTest failed\n");
	return -1;
    }

    do{
	if( ip_ptr != NULL )
	    delete ip_ptr;
	if( (ip_ptr = pos->second) == NULL ){
	    printDebug(3, "iperfResults_EndOfTest - no iperfResults\n");
	    return -1;
	}
        temp = ip_ptr->getEndTime();
        ( temp > endTime )? 
	    endTime = temp: endTime;
        ( ip_ptr->getTotalLen() > 0  )? 
	    bytesLen += ip_ptr->getTotalLen(): bytesLen;

        ip_Map.erase(pos);
    } while( (pos=ip_Map.find(group_id)) != ip_Map.end() );

    ip_ptr->setSocket(-1);
    ip_ptr->setStartTime(0.0);
    ip_ptr->setEndTime(endTime);
    ip_ptr->setTotalLen(bytesLen);

#if USE_NETLOGGER
    if( isNLenabled && isNLiperf ){
       RefreshNetLoggerHandle();
        if( Nlh != NULL ){
    	    printDebug(3, "6");
            ip_ptr->netLogResults_ulm(Nlh);
    	    printDebug(3, "7");
	}
        else
            cerr << "Error: netlogger handle is NULL\n";
    }
#endif

    if( isPrintIperf ){
        ip_ptr->printResults();
    }

    delete ip_ptr;
    return 0;
}


/* ---------------------------------------------------------
 * iperfResults_Jitter: 
 *	Called by PerfSocket
 *
 * --------------------------------------------------------- */

void         
Results::iperfResults_Jitter( IperfResults& ipR )
{
    IperfResults *ipR_ptr  = new IperfResults( ipR );

    if( isPrintIperf )
        ipR_ptr->printResults_Jitter();

    if( !isFinalReport ){
	printDebug(3, "iperfResults_Jitter - !isReportEndOf\n");
#if USE_NETLOGGER
        if( isNLenabled && isNLiperf ){
	    RefreshNetLoggerHandle();
            if( Nlh != NULL )
            	ipR_ptr->netLogResults_Jitter_ulm(Nlh);
            else
                cerr << "Error: NetLogger handle is NULL\n";
        }
#endif
    }
    else{
	printDebug(3, "iperfResults_Jitter - isReportEndOf\n");
        pthread_t groupID = ipR_ptr->getGroupID();
        ip_Map.insert( std::make_pair(groupID,ipR_ptr) );
    }
    return;
}


/* ---------------------------------------------------------
 * IperfResults_Jitter_EndOfTest: 
 *
 *	Called at end of test for multiple thread tests to get 
 *	a report of the total bits/s, bytes, jitter, drops, etc.
 *
 * --------------------------------------------------------- */

int
Results::iperfResults_Jitter_EndOfTest( pthread_t group_id )
{
    IperfResults *ip_ptr = NULL;
    IPMapi     pos;
    double     endTime=0.0,temp=0.0,jitter=0.0,jErr=0.0;
    int32_t    jOutOrd=0,jInDGs=0;
    max_size_t bytesLen=0;

    printDebug(3, "iperfResults_Jitter_EndOfTest\n");

    if( (pos=ip_Map.find(group_id)) == ip_Map.end() ){
        printDebug(3, "iperfResults_Jitter_EndOfTest failed\n");
	return -1;
    }

    do{
        if( ip_ptr != NULL )
            delete ip_ptr;
        if( (ip_ptr = pos->second) == NULL ){
            printDebug(3, "iperfResults_EndOfTest - no iperfResults\n");
            return -1;
        }
        temp = ip_ptr->getEndTime();
        ( temp > endTime )?
            endTime = temp: endTime;
        ( ip_ptr->getTotalLen() > 0  )?
            bytesLen += ip_ptr->getTotalLen(): bytesLen;
        ( ip_ptr->getJitter() > 0.0  )?
	    jitter += ip_ptr->getJitter(): jitter;
        ( ip_ptr->getJErrorcnt() > 0.0  )?
	    jErr += ip_ptr->getJErrorcnt(): jErr;
        ( ip_ptr->getJInOutofOrder() > 0  )?
	    jOutOrd += ip_ptr->getJInOutofOrder(): jOutOrd;
        ( ip_ptr->getJInDatagrams() > 0  )?
	    jInDGs += ip_ptr->getJInDatagrams(): jInDGs;
        ip_Map.erase(pos);
    } while( (pos=ip_Map.find(group_id)) != ip_Map.end() );

    ip_ptr->setSocket(-1);
    ip_ptr->setStartTime(0.0);
    ip_ptr->setEndTime(endTime);
    ip_ptr->setTotalLen(bytesLen);
    ip_ptr->setJitter(jitter);
    ip_ptr->setJErrorcnt(jErr);
    ip_ptr->setJInOutofOrder(jOutOrd);
    ip_ptr->setJInDatagrams(jInDGs);


#if USE_NETLOGGER
    if( isNLenabled && isNLiperf ){
        RefreshNetLoggerHandle();
        if( Nlh != NULL )
            ip_ptr->netLogResults_ulm(Nlh);
        else
            cerr << "Error: netlogger handle is NULL\n";
    }
#endif

    if( isPrintIperf )
        ip_ptr->printResults();

    delete ip_ptr;
    return 0;
}


/* ---------------------------------------------------------
 * tracerouteResults:
 *     assume all readings are ms
 *
 * stderr:
 *    1) test parameters: constant "target_host(), hops_max, byte_packets"
 *      traceroute to ux5.lbl.gov (128.3.7.103), 30 hops max, 40 byte packets
 * stdout:
 *    2)  output of 1st hop
 *    3)  output of 2nd hop
 *    4)  etc
 *      1  ir100gw-r2.lbl.gov (131.243.2.1)  0.94 ms   0.46 ms   0.43 ms
 *      2  ux5.lbl.gov (128.3.7.103)  0.65 ms   0.57 ms  *d7*
 *
 * ip - true  IP address
 *      false Canonical Names
 *
 * --------------------------------------------------------- */

#ifdef PATH_TRACEROUTE
void
Results::tracerouteResults( bool ip, string host, string parameters, string output )
{
    printDebug(3,"parse_traceroute_name: %s\n",output.c_str());
    if( isPrintSO )
        cout << output << endl;

#if USE_NETLOGGER
    if( !isNLenabled || !isNLSO ){
	return;
    }
    else{
	string addr1("(),");
	string d_data("*0123456789.");
	string d_label("*m");
	char   space(' ');
	string::size_type begIdx, endIdx;

	string target_host("DUMP");
	if(ip){
	    begIdx  = output.find_first_of( addr1[0], begIdx );  //  ( IP
	    endIdx  = output.find_first_of( addr1[1], begIdx );  //    IP )
	}
	else{
	    begIdx  = output.find_first_of( d_data[11], begIdx); // Canonical name '.'
	    begIdx  = output.find_last_of( space, begIdx);        // Start of Canonical name ' '
	    endIdx  = output.find_first_of( addr1[0], begIdx );  // '(IP)'
	}
	++begIdx;
	--endIdx;
	target_host.assign( output, begIdx, (endIdx-begIdx) );
	printDebug(3,"target_host = %s\n", target_host.c_str());

	// Parameter from first line: " 30 hops max"
	begIdx = output.find_first_of( addr1[2], 0 ); // first comma
	begIdx = output.find_first_of( d_data.substr(1,10), begIdx ); // first number
	endIdx = output.find_first_not_of( d_data.substr(1,10), begIdx ); // first non-number
	string hops_max;
	hops_max.assign( output, begIdx, (endIdx-begIdx));
	printDebug(3,"hops max    = %s\n", hops_max.c_str());

	// Parameter from first line: "41 byte packets"
	begIdx = output.find_first_of( addr1[2], endIdx ); // second comma
	begIdx = output.find_first_of( d_data.substr(1,10), begIdx ); // first number
	endIdx = output.find_first_not_of( d_data.substr(1,10), begIdx ); // first non-number
	string byte_packets;
	byte_packets.assign( output, begIdx, (endIdx-begIdx));
	printDebug(3, "byte packets= %s\n",byte_packets.c_str());

	//Parameter from second line
	char hop_counter[4]={space,'1',space,'\0'};
	begIdx  = output.find( hop_counter, endIdx);

	if(ip){
	    begIdx  = output.find_first_of( addr1[0], begIdx );  //  ( IP
	    endIdx  = output.find_first_of( addr1[1], begIdx );  //    IP )
	}
	else{
	    begIdx  = output.find_first_of( d_data[11], begIdx); // Canonical name '.'
	    begIdx  = output.find_last_of( space, begIdx);        // Start of Canonical name ' '
	    endIdx  = output.find_first_of( addr1[0], begIdx );  // '(IP)'
	}

	if( Nlh!=NULL ){
	    RefreshNetLoggerHandle();
	    if( isNLSO ){
		if( NetLoggerWrite(Nlh, "traceroute ", const_cast<char*>(target_host.c_str()),
				   const_cast<char*>(parameters.c_str())) < 0 )
		    cerr << "ERROR: NetLoggerWrite  1) Results\n";
		if( NetLoggerWrite(Nlh, "HopsMax", "HOPS=%s",  hops_max.c_str()) < 0 )
		    cerr << "ERROR: NetLoggerWrite  2) Results\n";
		if( NetLoggerWrite(Nlh, "PacketSize", "PKT.BYTE=%s", byte_packets.c_str(), host.c_str()) < 0 )
		    cerr << "ERROR:NetLoggerWrite  3) Results\n";
	    }
	    else
		printDebug(3,"SHOULD RETURN FAILED\n");
	}
	string::size_type tmpIdx, hopIdx=0;
	string bytes("bytes");

	do{
	    --endIdx;
	    ++begIdx;
	    if( begIdx == string::npos ){
                printDebug(3, "ERROR: no address");
		break;
	    }
	    else{
	        ++hop_counter[1];
		string  hop_address;
		string  data;
		string  num_bytes;
		//Hop Address
		hop_address.assign( output, begIdx, (endIdx-begIdx+1) );
		printDebug(3, "hop_address = %s\n",hop_address.c_str());

		if(!ip)
		    endIdx = output.find_first_of( addr1[1], endIdx );  // ')'

		// Data section
		//endIdx = output.find_first_of( addr1[1], endIdx );  // ')'
		begIdx = output.find_first_not_of( space, endIdx+1 ); // 0.70
		endIdx = output.find_first_of(  space, begIdx ); //0.70
		printDebug(3, "hop_counter[1] = %s\n", hop_counter);

		tmpIdx = output.find( bytes, hopIdx ); //If verbose 'bytes'
		hopIdx = output.find( (string)hop_counter, endIdx ); //end of a hop's output
		if( hopIdx!=string::npos )
		    num_bytes.assign(output, hopIdx, 3);

		if( tmpIdx!=string::npos ){
		    tmpIdx = output.find_first_of( d_data[11], tmpIdx ); //.
		    tmpIdx = output.find_first_of( space, tmpIdx );
		    begIdx = output.find_first_not_of( space, tmpIdx );
		    endIdx = output.find_first_of( space, begIdx );
		    string temp;
		    temp.assign(output,begIdx,endIdx-begIdx);
		    printDebug(3, "tmpIdx: %s\n",temp.c_str());
		}

		string temp;
		temp.assign(output,begIdx,(begIdx-endIdx));

		for(int i=0;endIdx!=string::npos&&endIdx<hopIdx; i++){
	       	    data.assign( output, begIdx, (endIdx-begIdx));
		    printDebug(3, "%d) %s\n",(i+1),data.c_str());

		    if( Nlh!=NULL ){
    		        if( isNLSO ){
    		    	    if( NetLoggerWrite(Nlh, "RoundTripTime", "RTT.MSEC=%s HOP_ADDR=%s", 
					       data.c_str(), hop_address.c_str() ) < 0 )
				cerr << "ERROR:NetLoggerWrite  3) Results\n";
			}
		    }
		    else
		        printDebug(3, "SHOULD RETURN Failed\n");

		    if( output[endIdx+1] == d_label[1] ) 
			endIdx+=3; // 'm'
		    begIdx = output.find_first_not_of(  space, endIdx ); //0.70
		    endIdx = output.find_first_of(  space, begIdx ); //0.70
		}
	    }
	    if(ip){
		begIdx  = output.find_first_of( addr1[0], begIdx );  //  ( IP
		endIdx  = output.find_first_of( addr1[1], begIdx );  //    IP )
	    }
	    else{
		begIdx  = output.find_first_of( d_data[11], hopIdx ); // Canonical name '.'
		begIdx  = output.find_last_of(  space, begIdx );       // Start of Canonical name ' '
		endIdx  = output.find_first_of( addr1[0], begIdx );  // '(IP)'
	    }
	} while( endIdx != string::npos );
    }
#endif /* USE_NETLOGGER */
    return;
}
#endif /* PATH_TRACEROUTE */


/* ---------------------------------------------------------
 *   pipecharResults:
 *
 * --------------------------------------------------------- */

#ifdef PATH_PIPECHAR
void
Results::pipecharResults( bool ip, CoordInfo &coordinfo, string output )
{
    string data;
    string host;
    int begIdx, endIdx;
    char   space(' ');

    printDebug(3,"parse_pipechar_name: %s\n",output.c_str());
    if( isPrintSO )
        cout << output << endl;

	// parse out the host name
	host.assign(coordinfo.getTestReceiver());

	// parse out the bottleneck speed
	begIdx = output.find("bottleneck");
	if (begIdx < 0 )
	{
		cerr << "ERROR:can't find bottleneck speed\n";
		return;
	}
	
	begIdx  = output.find( space , begIdx );
	endIdx  = output.find( space, begIdx+1);
	data.assign(output, begIdx+1, endIdx-begIdx-1);

#if USE_NETLOGGER
	RefreshNetLoggerHandle();
        int r = NetLoggerWrite(Nlh, "Pipechar",
                               "BOTTLENECK=%s NL.SND=%s NL.RCV=%s",
	                       data.c_str(),
		               coordinfo.getTestSender().c_str(),
		               coordinfo.getTestReceiver().c_str());
        if ( r < 0 )
	   cerr << "ERROR: NetLoggerWrite returned " << r << endl;
        else
	{
           cout << "NetLogger write returned " << r << endl;
	   NetLoggerFlush(Nlh);
	}
#endif /* USE_NETLOGGER */
	cout << "Pipechar bottleneck is " <<  data << " on path to " << coordinfo.getTestSender() << endl;
	cout << flush;

    return;
}
#endif /* PATH_PIPECHAR */
