/* -*-C++-*- */
/* Copyright (c) 1999 Regents of the University of California.
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <sys/types.h>
#include <termios.h>
#include <signal.h>
#include <errno.h>
#include <cerrno>

#ifdef sun
#include <sys/systeminfo.h>
#endif

#ifdef linux
#include <sys/utsname.h>
#endif

#include <sys/socket.h> //AF_INET
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h> //inet_ntoa
#include <sysexits.h>
#include <unistd.h>     // for close() with CC compiler
#include <sys/time.h>


#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN  256
#endif


#if HAVE_STREAMBUF
#include "sstream.h"
using std::ostringstream;
#else
#include <strstream>
using std::ostrstream;
#endif

#include "Display_funcs.h"
#include "funcs.h"

using std::cerr;
using std::endl;
using std::ifstream;

using std::cout;
using std::endl;

#define MAX_PASS_LEN 20


/* ---------------------------------------------------------------
 *  interface_Input: If want to input passwords from interface use
 *	set isInterface to 1;
 *
 * -------------------------------------------------------------*/

int 
interface_Input( bool isIn )
{
   static const int isInterface( ( isIn )?  1: 0 );
   return isInterface;
}

/* ---------------------------------------------------------------
 *  passwdS_cb: Password callback function for Server.  passwd is set the first
 *	time through.  Can be initialized by setting buff to NULL.
 *  - temp_passwdS: Is used to store the password when entered on
 *	the commandline, before passwdS_cb is called.
 *
 * -------------------------------------------------------------*/

int   	
passwdS_cb_Wrap( char* buff, int num, int verify, void* datatoPassBack )
{
    return passwdS_cb( buff, num, verify, datatoPassBack );
}

int   	
passwdS_cb( char* buff, int num, int verify, void* datatoPassBack )
{
   char   buf[MAX_PASS_LEN + 1];

   string prompt = "For nettest.pem, key.pem :\n\tEnter a Passphrase to decrypt private key: ";

   static const string passwd( (interface_Input(true))? 
				   getpass( buf, prompt.c_str() ): 
				   temp_passwdS("") );

   if( buff == NULL )
   	return 0;	

   strcpy( buff, passwd.c_str() );
   return strlen( buff );
}

string
temp_passwdS( const string pw )
{
    static string passwd = pw; 
    interface_Input(false);
    return passwd;
}


/* ---------------------------------------------------------------
 *  passwdC_cb: Password callback function for Client.  passwd is set the first
 *	time through.  Can be initialized by setting buff to NULL.
 *  - temp_passwdC: Is used to store the password when entered on
 *	the commandline, before passwdC_cb is called.
 *
 * -------------------------------------------------------------*/

int   	
passwdC_cb_Wrap( char* buff, int num, int verify, void* datatoPassBack )
{
    return passwdC_cb( buff, num, verify, datatoPassBack );
}

int   	
passwdC_cb( char* buff, int num, int verify, void* datatoPassBack )
{
   char   buf[MAX_PASS_LEN + 1];

   string prompt = "For Client cert/key pair ( entered on Command Line ):\n\tEnter a Passphrase to decrypt private key: ";

   static string passwd( (interface_Input(true))? 
				getpass( buf, prompt.c_str() ): 
				temp_passwdC("") );

   if( buff == NULL )
	return 0;	

   strcpy( buff, passwd.c_str() );
   return strlen( buff );
}

string
temp_passwdC( const string pw )
{
    static string passwd = pw;
    interface_Input(false);
    //if( passwd.size() == 0 )
    //    passwd = pw;

    return passwd;
}
	
/* ---------------------------------------------------------------
 *  getpass:
 *
 * -------------------------------------------------------------*/


string
getpass( char* buf, const char* prompt )
{
    string      passwd;
    char	*ptr;
    sigset_t	sig, sigsave;
    struct termios term, termsave;
    FILE	*fp;
    int		c;

    memset( buf, strlen(buf), 0 );

    if( (fp = fopen(ctermid(NULL), "r+")) == NULL )
	return NULL;
    setbuf( fp, NULL );

    sigemptyset( &sig );
    sigaddset( &sig, SIGINT );
    sigaddset( &sig, SIGTSTP );
    sigprocmask( SIG_BLOCK, &sig, &sigsave );

    tcgetattr( fileno(fp), &termsave );
    term = termsave;
    term.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
    tcsetattr( fileno(fp), TCSAFLUSH, &term );

    fputs( prompt, fp );
    
    ptr = buf;
    while( (c=getc(fp)) != EOF && c != '\n' )
    {
	if( ptr < &buf[MAX_PASS_LEN] )
	    *ptr++ = c;
    }

    *ptr = 0;
    putc('\n', fp);
    tcsetattr( fileno(fp), TCSAFLUSH, &termsave );
    sigprocmask( SIG_SETMASK, &sigsave, NULL );
    fclose( fp );
    passwd =  buf;

    return passwd;
}


/* -----------------------------------------------------------------
 * getLocalHostName: Returns the canonical name of the local host
 *
 * -----------------------------------------------------------------*/

int
getLocalHostName( string& name )
{
    char buf[MAXHOSTNAMELEN];
    memset( buf, 0, MAXHOSTNAMELEN );

#ifdef sun
    sysinfo(SI_HOSTNAME, buf, (long)MAXHOSTNAMELEN);
#else
    gethostname(buf,MAXHOSTNAMELEN);
#endif

    name = buf;
    if( getCanonicalName(name) < 0 )
        return -1;

    return 0;
}

string
getLocalHostName( void )
{
    string l_host;
    getLocalHostName( l_host );
    return l_host;
}


/* ---------------------------------------------------------------
 *  getCanonicalName:  Returns the Canonical name of the host past
 *	in.  Thread Safe reentrant routines used.
 *
 * -------------------------------------------------------------*/

int getCanonicalName( string& buf )
{
    if( buf.empty() ) 
	return -1;

    int    err;
    extern int errno;
    struct hostent *hp=NULL;

    struct hostent result;
    char   buffer[8192];

#ifdef linux
    if( isdigit(buf.at(0)) )
    {
        unsigned long addr = inet_addr( buf.c_str() );
        if( gethostbyaddr_r( (char*)&addr, sizeof(addr), AF_INET, 
			      &result, buffer, sizeof(buffer), &hp, &err ) < 0 )
	    hp = NULL;
    }
    else
    {
        if( gethostbyname_r( buf.c_str(), &result, buffer, 	
			     sizeof(buffer), &hp, &err ) < 0 )
	    hp = NULL;
    }
#else
    if( isdigit(buf.at(0)) )
    {
        unsigned long addr = inet_addr( buf.c_str() );
        hp = gethostbyaddr_r( (char*)&addr, sizeof(addr), AF_INET, 
			      &result, buffer, sizeof(buffer), &err );
    }
    else
        hp = gethostbyname_r( buf.c_str(), &result, buffer, sizeof(buffer), &err );
#endif
    // ?? Problem with RedHat7
    if( hp == NULL )
    {
        perror("getCanonicalName failed\n");
        return -1;
    }

    buf = result.h_name;

    return 0;
}

/* -----------------------------------------------------------------
 * int_to_string: Takes an int and returns it as a string.
 *
 * -----------------------------------------------------------------*/

string
int_to_string( int number )
{
    string result;

#if HAVE_STREAMBUF
    ostringstream os;
#else
    ostrstream os;
#endif


    os << number;
    result = os.str();

    return result;
}

/* -----------------------------------------------------------------
 * double_to_string: Takes an double and returns it as a string.
 *
 * -----------------------------------------------------------------*/

string
double_to_string( double number )
{
    string result;

#if HAVE_STREAMBUF
    ostringstream os;
#else
    ostrstream os;
#endif

    os << number;
    result = os.str();

    return result;
}


/* -----------------------------------------------------------------
 * float_to_string: Takes an float and returns it as a string.
 *
 * -----------------------------------------------------------------*/

string
float_to_string( float number )
{
    string result;

#if HAVE_STREAMBUF
    ostringstream os;
#else
    ostrstream os;
#endif

    os << number;
    result = os.str();

    return result;
}

/* -----------------------------------------------------------------
 * getToken:  Returns an empty string if one requests an
 *            invalide token.
 *
 * -----------------------------------------------------------------*/

string
getToken( const string line, const string delimiters, const int tokenNum )
{
    string tokenStr;
    string::size_type begIdx, endIdx;
    int count = tokenNum;

    begIdx = line.find_first_not_of( delimiters, 0 );

    for( int i=0; i<count; i++ )
    {
        begIdx = line.find_first_of( delimiters, begIdx );
        begIdx = line.find_first_not_of( delimiters, begIdx );
    }

    if( begIdx == string::npos )
        return "";

    endIdx = line.find_first_of( delimiters, begIdx );

    if( endIdx == string::npos )
        endIdx = line.length();

    tokenStr.assign( line, begIdx, (endIdx-begIdx) );

    return tokenStr;
}
