/* 
 * Copyright (c) 1999 Regents of the University of California.
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley 
 *       National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: network.cc,v 1.15 2001/10/15 21:23:46 boverhof Exp $
*/

#include "network.h"
#include "headers.h"
#include <cstring>
#include <cerrno>
#include <iostream>


using namespace std; 
//using std::string;
//using std::strcpy;
//using std::perror;
//using std::printf;
//using std::memset;
//using std::errno;

//extern int h_errno;

pthread_mutex_t Network::rmutex;  // socket receive mutex
pthread_mutex_t Network::smutex;  // socket send mutex

/***********************************************************************
 *  Network - constructor
 *  arguments
 *      fd - socket file descriptor
 *      s  - number of seconds to block on calls to doRecv
 *      p  - port used by this socket
 */

Network::Network(int fd, int s, short p):  MyDebug("Network: ")
{
    printDebug(5, "Network Initialize\n");
    sock = fd;
    block_secs = s;
    block_usecs = 0;
    port = p;
    memset(&peer, 0, sizeof(peer));	
    int status = pthread_mutex_init(&rmutex, NULL);
    assert(status == 0);	
    status = pthread_mutex_init(&smutex, NULL);
    assert(status == 0);	
}

/***********************************************************************
 *  Network object destructor
 */

Network::~Network()
{
    if( sock > 0 )
        close(sock);
}

/***********************************************************************
 *  setPort - sets the port
 *  argument 
 *     p - new port 
 */

void Network::setPort(short p)
{
    port = p;
}

/***********************************************************************
 *  setBlockTime -
 *  Sets the amount of time that we should block on each call to doRecv.
 *
 *  arguments
 *       secs - number of seconds to block
 *       usecs - number of microseconds to block
 */

void Network::setBlockTime( int secs, int usecs )
{
    block_secs = secs;
    block_usecs = usecs;
    printExactDebug( 2, "Setting blocking time to %d seconds and %d usecs\n", secs, usecs );
    return;
}


/*********************************************************************** 
 * getLocalName: Get name of local end of the socket.  
 *
 * Return:
 *   ptr to name, or NULL if there was an error
 */
#if 0
string*
Network::getLocalName(void)
{
   struct sockaddr_in local_name;
   u_int32_t local_len;
   struct hostent *local_host_p;
   int r;

   local_len = sizeof(local_name);
   r = getsockname( sock, (struct sockaddr*)&local_name, &local_len );
   if ( r != 0 )
   { 
        perror("getsockname");
        return NULL;
   }

   // Debugging
   fprintf(stderr,"Local address: %08x\n", local_name.sin_addr.s_addr);

   local_host_p = gethostbyaddr( (const char*)&local_name.sin_addr,
                                 sizeof(local_name.sin_addr), AF_INET);
   if ( local_host_p == NULL )
   { 
        cerr << "gethostbyaddr returned NULL\n";
        herror("gethostbyaddr");
        return NULL;
   }

   // Debugging
   fprintf(stderr,"Got host name: %s\n",local_host_p->h_name);

   return new string(local_host_p->h_name);
}
#endif /* 0 */
#if 1
/*********************************************************************** 
 *  getLocalname -
 *  get the name of the local host and return it
 *  This routine will return the ASCII dotted notation name.
 *  return NULL if fail
 *
 *   sizeof buffer? 8192
 */

string* Network::getLocalName(void)
{
    char buf[MAXHOSTNAMELEN];
    string* ret_p = NULL;

#ifdef sun 
    int    err;
    struct hostent *hp = NULL;
    struct hostent result;   //  allocated by the caller
    char   buffer[8*MAXHOSTNAMELEN];     //  allocated by the caller

    sysinfo(SI_HOSTNAME, buf, MAXHOSTNAMELEN);
    hp = gethostbyname_r(buf,&result,buffer,sizeof(buffer),&err);
    if ( hp ){
	ret_p = new string(hp->h_name);
        printDebug(5, "getLocalname : ret_p = %s\n", ret_p);
    }
    else
        printDebug(5, "getLocalname : gethostbyname_r Error\n");
#else
    struct sockaddr_in localaddr;
    memset( &localaddr, 0, sizeof(localaddr) );
    if( gethostname(buf, MAXHOSTNAMELEN) == 0 ){
	ret_p = new string(buf);
        printDebug(5, "getLocalname : ret_p = %s\n", ret_p);
    }
    else
        printDebug(5, "getLocalname : gethostname Error\n");
#endif  

    return ret_p;
}
#endif /* 1 */

/***********************************************************************
 *  getRemoteHost -
 *  Extract the remote host name from the
 *  sock addr struct for this connection and 
 *  convert it to the dotted character notation
 *  and sets the value in the character buffer
 *  passed in. 
 *     
 *  argument
 *     host - buffer to write the remote host name to
 *  not thread safe
 *   //	name = new string(inet_ntoa(peer.sin_addr)); 
 */
	
string* Network::getRemoteHost(void)
{
    //extern int errno;
    int    err;
    struct hostent result; // allocated by caller
    char   buffer[8*MAXHOSTNAMELEN];

    string* name = NULL;
    struct hostent *hp=NULL;

#ifdef sun
    hp = gethostbyaddr_r( (char *)&peer.sin_addr,sizeof(peer.sin_addr),AF_INET,
			  &result, buffer, sizeof(buffer), &err );
#else  // linux 
    gethostbyaddr_r( (char *)&peer.sin_addr,sizeof(peer.sin_addr),AF_INET,
			  &result, buffer, sizeof(buffer), &hp, &err );
#endif

    if ( hp ){
    	name = new string(hp->h_name);
        printDebug(5, "getRemoteHost: %s\n", name);
    }
    else
        printDebug(5, "getRemoteHost: Error");

    return name;
}

/***********************************************************************
 *  doSend - 
 *  Send a message on the socket.
 *
 *  arguments
 *      msg - the message to send
 *      len - the length of the message to be sent
 */

int Network::doSend(const char *msg, long len)
{
    long n;

    if( pthread_mutex_lock(&smutex) != 0 )
	printExactDebug( 2, "Network::getRemoteHost--can't lock send mutex\n");

    n = sendto(sock, msg, len, 0, (struct sockaddr *)&peer, sizeof(peer));

    if( pthread_mutex_unlock(&smutex) != 0 )
	printExactDebug( 2, "Network::getRemoteHost--can't unlock send mutex\n");

    printExactDebug( 2, "Network::doSend--sending msg: %s\n",msg); 

    if (n < 0) 
      perror("Network: sendto()");
    else if (n < len) 
      cerr << "Network: sendto() didn't send whole message\n";
    return n; 	
}

/***********************************************************************
 *  doRecv - 
 *  Receive a message from the socket.  The receive will block waiting
 *  for the message the current block_sec and block_usec time.
 *
 *  arguments
 *      buf - the buffer to copy the  message into
 *      len - the length of the buffer passed in
 */

int Network::doRecv(char *buf, long len)
{
    // int l;
    Socklen_t l;
    int n;
    struct timeval t;      
    fd_set fd;

    t.tv_sec = block_secs;
    t.tv_usec = block_usecs;
    FD_ZERO(&fd);
    FD_SET(sock, &fd);
    n = select(sock+1, &fd, NULL, NULL, &t);
    if (n > 0)
    {
	l = sizeof(peer);
	n = recvfrom(sock, buf, len, 0, (struct sockaddr *)&peer, &l);
	printExactDebug( 2, "Network::doRecv--recvfrom returned %d\n",n);
	if (n < 0) 
	{	
	    printError("Network::doRecv: recvfrom() failed\n");	
	}
	else
	{
	     buf[n] = '\0';
	     n++;	
	}
    }

    if ( block_usecs > 0 || block_secs > 0 )
        printExactDebug( 2, "Network::doRecv--returning %d\n", n);
    return n;
}			
  
/***********************************************************************
 *  doBlockedRecv - 
 *  Receive a message from the socket.  The receive will block waiting
 *  for the message indefinitely or untill an error occurs on the socket.
 *
 *  arguments
 *      buf - the buffer to copy the  message into
 *      len - the length of the buffer passed in
 *      sender - buffer to copy the sender's id into
 */

int Network::doBlockedRecv(char *buf, long len, char *sender)
{
    /* This routine blocks until data arrives on the socket and
       extracts the sender's ip addr returned by recvfrom(). */ 	 
  
    int n;
    Socklen_t l;
    //int l;
    sender[0] = '\0';	
    l = sizeof(peer);
    n = recvfrom(sock, buf, len, 0, (struct sockaddr *)&peer, &l);
    if ( errno != EINTR )
        printExactDebug( 2, "Network::doBlockedRecv--recvfrom returned %d\n",n);
    if (n < 0)
    {	
        if (errno == EINTR)
	{
	    printExactDebug( 2, "recvfrom() returned EINTR\n");
	    return -2;
	}
	printExactDebug( 2, "doBlockedRecv: recvfrom() errno: %d\n", errno);
	printError("Network::doBlockedRecv: recvfrom() failed\n");	
    }
    else
    {
        buf[n] = '\0';
        n++;	
    }
    string* remote_p = getRemoteHost();	
    strcpy(sender, remote_p->c_str());
    delete remote_p;
    printExactDebug( 2, "Network::doBlockedRecv--returning %d\n", n);
    return n;
}			

/***********************************************************************
 *   convertAddr - 
 *   Takes in a string defining in dotted characters or
 *   dotted decimals a host name and then it returns
 *   the inet addr of this host (32 bit).
 */

unsigned long Network::convertAddr(const char *s)
{
    if (isdigit(*s)) // group_addr dotted decimal convert to 32-bit IP address.
        return (unsigned long)inet_addr(s);

    int    err;
    char   buffer[8*MAXHOSTNAMELEN];     //  allocated by the caller

    struct hostent result;   //  allocated by the caller
    struct hostent *hp = NULL;

#ifdef sun
    hp = gethostbyname_r(s,&result,buffer,sizeof(buffer),&err);
#else  // linux
    gethostbyname_r(s,&result,buffer,sizeof(buffer),&hp,&err);
#endif

    if ( !hp )
   	return 0;

    return *((unsigned long **)hp->h_addr_list)[0];
}

/***********************************************************************
 *  isClassDAddr - 
 *  Determines whether the address passed in is a class D (multicast)
 *  address.
 */

int Network::isClassDAddr(const char *addr)
{
    unsigned long ip_addr = convertAddr(addr);
    return CLASSD_ADDR(ip_addr); 
}

