/*
 *  Copyright (c) 2000 Regents of the University of California.
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley 
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 * 
 */

#include "config.h"
#include "IperfResults.h"
#include "IperfTestInfo.h"
#include "PerfSocket.hpp"
#include "Locale.hpp"
#include "util.h"
#include <cstdio>
#include <sys/types.h> // typedef pthread_t -> unsinged int

#if USE_NETLOGGER
#include "logging.h"
#endif

#include <iostream>

using std::cout;
using std::cerr;
using std::endl;

#define AVGWINDOW 16


/* -------------------------------------------------------------------
 *  IperfResults:
 *
 * ------------------------------------------------------------------- */

IperfResults:: IperfResults(PerfSocket& perfsock, const double start_time, 
	const double end_time, const max_size_t totalBytes)
    : MyDebug("IperfResults"),remoteHost("unspecified"), 
	localHost("unspecified"), rTestParam("xxx"), tBytes("xxx"), 
	tSpeed("xxx"), rTotalLen(totalBytes),  rStartTime(start_time), 
	rEndTime(end_time), jJitter(-1.0), jErrorcnt(-1), jInOutofOrder(-1), 
	jInDatagrams(-1), sock(-1), isUDP(false), groupID(0)
{
    fillStats(perfsock);
}

IperfResults::IperfResults(PerfSocket& perfsock, const double start_time, 
	const double end_time, const int32_t errorcnt, const int32_t outOrder, 
	const int32_t inDG, const max_size_t totalBytes ) 
    : MyDebug("IperfResults"),remoteHost("unspecified"), 
	localHost("unspecified"), rTestParam("xxx"), tBytes("xxx"), 
	tSpeed("xxx"), rTotalLen(totalBytes),  rStartTime(start_time), 
	rEndTime(end_time), jJitter(-1.0), jErrorcnt(errorcnt), 
	jInOutofOrder(outOrder), jInDatagrams(inDG), sock(-1), 
	isUDP(false), groupID(0)
{
    fillStats(perfsock);
}

IperfResults::IperfResults()
    : MyDebug("IperfResults"),remoteHost("unspecified"), 
	localHost("unspecified"), rTestParam("xxx"), tBytes("xxx"), 
	tSpeed("xxx"), rTotalLen(-1),  rStartTime(-1.0), rEndTime(-1.0), 
      	jJitter(-1.0), jErrorcnt(-1), jInOutofOrder(-1), jInDatagrams(-1), 
	sock(-1), isUDP(false), groupID(0)
{
}

IperfResults::IperfResults( const IperfResults& ipR )
{
    remoteHost = ipR.remoteHost;
    localHost  = ipR.localHost;
    rTestParam = ipR.rTestParam;
    mSender    = ipR.mSender;
    mReceiver  = ipR.mReceiver;
    tBytes     = ipR.tBytes;
    tSpeed     = ipR.tSpeed;
    rTotalLen  = ipR.rTotalLen;

    rStartTime = ipR.rStartTime;
    rEndTime   = ipR.rEndTime;

    jJitter    = ipR.jJitter;
    jErrorcnt  = ipR.jErrorcnt;
    jInOutofOrder = ipR.jInOutofOrder;
    jInDatagrams  = ipR.jInDatagrams;

    sock     = ipR.sock;
    bwFormat = ipR.bwFormat;
    isUDP    = ipR.isUDP;
    groupID  = ipR.groupID;
}

IperfResults::~IperfResults()
{
}


/* -------------------------------------------------------------------
 * fillStats
 *
 *  PerfSocket::ReportPeriodicBW(void)
 *  ReportBW( mTotalLen - mPLastTotalLen,
 *            mPLastTime.subSec( mStartTime ),
 *            mPacketTime.subSec( mStartTime ));
 * 
 * ------------------------------------------------------------------- */

void
IperfResults::fillStats(PerfSocket& psock)
{
   isUDP      = psock.ipSettings_p->getUDPFlag();
   remoteHost = ( psock.ipSettings_p->getRemoteTestPeer() );
   localHost  = ( psock.ipSettings_p->getLocalName() );
   rTestParam = ( psock.ipSettings_p->getTestParams() );
   bwFormat   = ( psock.ipSettings_p->getFormat());
   sock	      = psock.mSock;
  
   ( isUDP )? jJitter = (psock.mJitter)*1000.0: -1.0 ;

    if(psock.ipSettings_p->isSender()) {
        mSender   = localHost;
        mReceiver = remoteHost;
    }
    else {
        mSender   = remoteHost;
        mReceiver = localHost;
    }
    groupID = (psock.ipSettings_p->getGroupID());
    printDebug(3, "fillStats - groupID = %d\n",groupID);
}

/* -------------------------------------------------------
 *  printResults
 *
 * -------------------------------------------------------- */

void
IperfResults::printResults( void )
{
  double secs = rEndTime - rStartTime;
  char bytes[ 32 ];
  char speed[ 32 ];

  byte_snprintf( bytes, sizeof(bytes), rTotalLen, toupper( bwFormat ) );
  byte_snprintf( speed, sizeof(speed), rTotalLen / secs, bwFormat );

  printf( report_bw_format, sock, rStartTime, rEndTime, bytes, speed );
  fflush( stdout );
}

/* --------------------------------------------------------
 *  printResults_Jitter
 *
 * -------------------------------------------------------- */

void
IperfResults::printResults_Jitter( void )
{
  double secs = rEndTime - rStartTime;
  char bytes[ 32 ];
  char speed[ 32 ];

  byte_snprintf( bytes, sizeof(bytes), rTotalLen, toupper(bwFormat) );
  byte_snprintf( speed, sizeof(speed), rTotalLen / secs, bwFormat );

  printf( report_bw_jitter_loss_format,
          sock, rStartTime, rEndTime, bytes, speed,
          jJitter, jErrorcnt, jInDatagrams,
          (100.0 * jErrorcnt) / jInDatagrams );

  if ( jInOutofOrder > 0 ) {
    printf( report_outoforder, sock, rStartTime, rEndTime, jInOutofOrder );
  }
  fflush( stdout );

  return;
}


/* -------------------------------------------------------------------
 * netLogResults_ulm: Output results to NetLogger handle.  First
 *	parse out data and the format of the data from tBytes and tSpeed.
 *
 * ------------------------------------------------------------------- */

#if USE_NETLOGGER
void
IperfResults::netLogResults_ulm(struct _loginfo * Nlh)
{
#ifdef SENDER_LOGS_ONLY
    if ( !strcmp(mSender.c_str(),Nlh->hname) ) return;
#endif // SENDER_LOGS_ONLY

    double secs  = rEndTime - rStartTime;
    int    speed = static_cast<int>( 8*rTotalLen/secs ); 
    string testArgs(rTestParam);

    replaceSpaces(testArgs);

    string formatTest("ARGS=%s NL.SND=%s NL.RCV=%s SCK=%d STR.SEC=%.1f END.SEC=%.1f LEN.BYT=%ld SPD.BIT/S=%ld");
    int    tlen = static_cast<int>(rTotalLen);

    if(isUDP) {
        printDebug(3, "netLogResults isUDP\n");
        if( NetLoggerWrite(Nlh, "iperfUDP", const_cast<char*>(formatTest.c_str()), testArgs.c_str(), 
		mSender.c_str(), mReceiver.c_str(), sock, rStartTime, rEndTime, tlen, speed ) < 0 )
		cerr << "ERROR: NetLoggerWrite iperfUDP\n";
    }
    else {
        printDebug(3, "netLogResults isTDP\n");
        if( NetLoggerWrite(Nlh, "iperfTCP", const_cast<char*>(formatTest.c_str()), testArgs.c_str(), 
	    	mSender.c_str(), mReceiver.c_str(), sock, rStartTime, rEndTime, tlen, speed ) < 0 )
		cerr << "ERROR: NetLoggerWrite iperfTCP\n";
    }
}


/* -------------------------------------------------------------------
 * netLogResults_Jitter_ulm: Output results to NetLogger handle.  First
 *      parse out data and the format of the data from tBytes and tSpeed.
 *
 * ------------------------------------------------------------------- */

void
IperfResults::netLogResults_Jitter_ulm(struct _loginfo * Nlh)
{
#ifdef SENDER_LOGS_ONLY
    if ( !strcmp(mSender.c_str(),Nlh->hname) ) return;
#endif // SENDER_LOGS_ONLY

    double  secs  = rEndTime - rStartTime;
    int     speed = static_cast<int>(8*rTotalLen/secs);
    string testArgs(rTestParam);
    replaceSpaces(testArgs);
    string formatTest("ARGS=%s NL.SND=%s NL.RCV=%s SCK=%d STR.SEC=%.1f END.SEC=%.1f LEN.BYT=%ld SPD.BIT/S=%ld JI.MSEC=%.3f ERR.CNT=%ld DATAGRM=%ld OUT.ORD=%ld");

    printDebug(3, "netLogResults_jitter isUDP\n");
    if( NetLoggerWrite(Nlh, "iperfUDP", const_cast<char*>(formatTest.c_str()), testArgs.c_str(), mSender.c_str(), mReceiver.c_str(), 
	sock, rStartTime, rEndTime, static_cast<int>(rTotalLen), speed, jJitter, jErrorcnt, jInDatagrams, jInOutofOrder  ) < 0 )
	    cerr << "ERROR:NetLoggerWrite  iperfUDP Jitter\n";
    return;
}

/* ---------------------------------------------------------------------------
 * replaceSpaces: Replaces all instances of ' ' with '_', and removes spaces at
 *      head and tail of string.
 *
 * ---------------------------------------------------------------------------*/

void
IperfResults::replaceSpaces( string& raw )
{
    char space(' ');
    string::size_type begIdx = raw.find_first_not_of( space );
    string::size_type endIdx = raw.find_last_not_of( space );

    if ( begIdx != string::npos ){
        char us('_');
        for( string::size_type i=begIdx; i<=endIdx; i++ ){
            if( raw[i]==space )
                raw[i] = us;
        }
        raw = raw.substr(begIdx, endIdx-begIdx+1);
    }
    else
        raw = "???";
}
#endif
