/* -*-C++-*- */
/*
 *  Copyright (c) 2000 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 */


#include "IperfTestInfo.h"
#include "Results.h"
#include "headers.h"
//#include "Locale.hpp"
#include "util.h"
#include <iostream>
#include <cstring>
#include <string>
//#include "stdio.c"

using std::cerr;
using std::endl;

/* -------------------------------------------------------------------
 * defaults
 * ------------------------------------------------------------------- */
#define DEFAULTS()

const long kDefault_UDPRate = 1024 * 1024; // -u  if set, 1 Mbit/sec
const int  kDefault_UDPBufLen = 1470;      // -u  if set, read/write 1470 bytes
// 1470 bytes is small enough to be sending one packet per datagram on ethernet


/* -------------------------------------------------------------------
 *  IperfTestInfo
 *
 * ------------------------------------------------------------------- */

IperfTestInfo::IperfTestInfo( void )
    : TestInfo(false, string(""), string(""), 5001, 8192, 50.0, Timed, 10.0), 
      mUdpRate(kDefault_UDPRate), mAmount(0.0), mInterval(5.0), mWinSize(0), mMaxSegSize(0), 
      mMcastTtl(1), mThreads(1), mMcastAddr(""), mFormat('a'),mTOS(0),mIsUdp(false), 
      mPrintMSS(false), mIsNoDelay(false), groupID(0)
{
    setClassName( "IperfTestInfo: ");
}

IperfTestInfo::IperfTestInfo( const TestInfo& curinfo )
    : TestInfo( curinfo ), 
      mUdpRate(kDefault_UDPRate),mAmount(0.0), mInterval(5.0),mWinSize(0),mMaxSegSize(0), 
      mMcastTtl(1), mThreads(1), mMcastAddr(""),mFormat('a'),mTOS(0),mIsUdp(false), 
      mPrintMSS(false), mIsNoDelay(false), groupID(0)
{
    setClassName( "IperfTestInfo: ");
}

IperfTestInfo::~IperfTestInfo(void)
{
}


/* -------------------------------------------------------------------
 * printVariables
 *
 * ------------------------------------------------------------------- */

ostream &
IperfTestInfo::printVariables(ostream & stream) 
{
    TestInfo::printVariables(stream);
    stream << "MaxSegSize = " << mMaxSegSize << endl;
    stream << "Winsize = " << mWinSize << endl;
    stream << "noDelay = " << mIsNoDelay << endl;
    stream << "mcastTTL = " << mMcastTtl << endl;
    stream << "udpRate = " << mUdpRate << endl;
    stream << "mcastAddr = " << mMcastAddr << endl;
    stream << "TOS = " << mTOS << endl;
    stream << "format = " << mFormat << endl;
    stream << "udpFlag = " << mIsUdp << endl;
    stream << "printMSS = " << mPrintMSS << endl;
    stream << "mThreads = " << mThreads << endl;
    stream << "groupID = " << groupID << endl;
    return stream;
}


/* -------------------------------------------------------------------
 *  setUserParams:  Set Iperf Test Parameters.  mTest_Parms must be set
 *	before entering.
 *
 * ------------------------------------------------------------------- */

void
IperfTestInfo::setUserParams( void )
{
    string tmpParam( mTest_Params );
    string tokenDelims( "-" );
    string tokenNum( "0123456789KM." );
    string tokenForm( "aAbBkKmM" );
    string number;
    string::size_type begIdx;

    printDebug(2, "SetUserParams params: %s\n", tmpParam.c_str());

    begIdx = tmpParam.find_first_of( tokenDelims, 0 );
    if( begIdx == string::npos )
        return;

    printDebug(2, "SetUserParams while loop\n");
    getResults()->setIsReportEndOfTest(false);

    while( true )
    {
	begIdx = tmpParam.find_first_not_of( tokenDelims, begIdx );
	if( begIdx == string::npos )
	    return;

	switch ( tmpParam[begIdx] )
	{
	    // CLIENT & SERVER OPTIONS *****************************
	    case 'd': // delay in seconds
		getToken( number, tmpParam, tokenNum, begIdx );
		setDelay(atoi(number.c_str()));
		break;
	    case 'f': // format to print bandwidth
		getToken( number, tmpParam, tokenForm, begIdx );
		mFormat = number[0];
		break;
	    case 'i': // specify interval between periodic bw reports
		getToken( number, tmpParam, tokenNum, begIdx );
		mReportInterval = atof(number.c_str());
		break;
	    case 'l':
		getToken( number, tmpParam, tokenNum, begIdx );
		//setBufferLen(atoi(number.c_str()));
		setBufferLen( static_cast<int>(byte_atof(number.c_str())) );
		mBufLenSet = true;
		break;
	    case 'm': // Print reported TCP MSS size
		mPrintMSS = true;
		break;
	    case 'p': // server port for server to listen, client to connect
		getToken( number, tmpParam, tokenNum, begIdx );
		setPort(atoi(number.c_str()));
		break;
	    case 'u': // use UDP
		mIsUdp = true;
		if ( mUdpRate == 0 ) {
		    mUdpRate = kDefault_UDPRate;
		}
		// if -l has already been processed, mBufLenSet is true
		// so don't overwrite that value.
		if ( ! mBufLenSet ) {
		    mBufferLen = kDefault_UDPBufLen;
		}
		break;
	    case 'w': // TCP window size (socket buffer size)
		getToken( number, tmpParam, tokenNum, begIdx );
		//mWinSize = atoi(number.c_str());
		mWinSize = static_cast<int>(byte_atof(number.c_str()));
		if ( mWinSize < 2048 ) {
		    cerr << "Window too small: "<<mWinSize<<endl;
		    cerr << "Set window size to 2048\n";
		    mWinSize = 2048;
		    //printf( warn_window_small, mWinSize );
		}
		break;
	    //case 'B': // Bind to host
		//getToken( mLocalname, tmpParam, tokenNum, begIdx );
		//mLocalhost = new char[ strlen(optarg) + 1 ];
		//strcpy( mLocalhost, optarg );
		//break;
	    case 'M': // specify TCP MSS (maximum segment size)
		getToken( number, tmpParam, tokenNum, begIdx );
		//mMaxSegSize = (long) atoi(number.c_str());
		mMaxSegSize = static_cast<int>(byte_atof(number.c_str()));
		break;
	    case 'N': // set the TCP no delay option
		mIsNoDelay = true;
		break;
		// SERVER SPECIFIC OPTIONS *****************************
		// CLIENT SPECIFIC OPTIONS *****************************
	    case 'b': // UDP bandwidth to send at in bits/sec, implies -u option
		getToken( number, tmpParam, tokenNum, begIdx );
		mUdpRate = atof(number.c_str());
		// if -l has already been processed, mBufLenSet is true
		// so don't overwrite that value.
		if ( ! mBufLenSet ) {
		    mBufferLen = kDefault_UDPBufLen;
		}
		break;
	    case 'n': // bytes of data
		// positive indicates amount mode (instead of time mode)
		getToken( number, tmpParam, tokenNum, begIdx );
		mTermination = NumBytes;
		setTestAmount(byte_atof(number.c_str()));
		//setTestAmount(atoi(number.c_str()));
		break;
	    case 't':
		getToken( number, tmpParam, tokenNum, begIdx );
		mTermination = Timed;
		setTestAmount(atoi(number.c_str()));
		break;
	    case 'P': // number of client threads
		getToken( number, tmpParam, tokenNum, begIdx );
		mThreads = atoi(number.c_str());
		break;
	    case 'S': // type-of-service for outgoing packets.
		mTOS = strtol( optarg, NULL, 0 );
		break;
	    //case 'T': // time-to-live for multicast
	//	getToken( number, tmpParam, tokenNum, begIdx );
	//	mMcastTtl = atoi(number.c_str());
	//	break;
#ifdef NETTEST
	    case 'e': // print only cumulative stats at the end-of-test
		getResults()->setIsReportEndOfTest(true);
	 	break;
#endif /* NETTEST */
	    default:
		cerr<<"Error Incorrect parameter: "<< tmpParam[begIdx] <<endl;
		break;
	}

        begIdx = tmpParam.find_first_of( tokenDelims, begIdx );
        if( begIdx == string::npos )
            break;
    }

    return;
} 
