/*
 *  Copyright (c) 2000 Regents of the University of California.
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley 
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 * 
 *  $Id: PingResults.cc,v 1.26 2001/08/30 02:49:41 boverhof Exp $
 */

#include "PingResults.h"
#include "PingTestInfo.h"
#include "Results.h"

#if USE_NETLOGGER
#  include "logging.h"
#endif

#include <iostream>

using std::cout;
using std::cerr;
using std::endl;

#define AVGWINDOW 16

/* ---------------------------------------------------------
 *  PingResults: Storage class for UDP ping and 
 *		 multicast RTT tests.
 *
 * --------------------------------------------------------- */

PingResults::PingResults ( PingTestInfo pingInf  )
    : MyDebug("PingResults"), mMaxrtt(-1.0), mMinrtt(-1.0), mAvertt(-1.0), mCurlast(-1.0), mMulticast(false)
{
    mTestParam = pingInf.getTestParams();
    mLocalhost = pingInf.getLocalName();
    mPeeraddr  = pingInf.getLocalTestInterface();
    mRemote    = pingInf.getRemoteTestPeer();
    mMulticast = pingInf.getIsMulti();

    if(pingInf.isSender()) { 
	mSender   = pingInf.getLocalName(); 
	mReceiver = pingInf.getRemoteTestPeer();
    }
    else {
	mSender   = pingInf.getRemoteTestPeer();
	mReceiver = pingInf.getLocalName(); 
    }
}

PingResults::~PingResults()
{
}


/* ---------------------------------------------------------
 *  fillStats
 *
 * --------------------------------------------------------- */

//void
//PingResults::fillStats( pthread_mutex_t& mStats_mutex, double mLastrtt[16], int mNumbertests)

void
PingResults::fillStats( double mLastrtt[16], int mNumbertests )
{
    int    index = 0;

    //pthread_mutex_lock( &mStats_mutex );

    index = (mNumbertests%AVGWINDOW) - 1;
    mAvertt = 0.0;

    int i;
    for( i=0; i< AVGWINDOW && i < mNumbertests ; i++ )
     {
	if( i == 0 )
	  {
	    mMaxrtt = mLastrtt[i];
	    mMinrtt = mLastrtt[i];
	  }
	else
	  {
	    if( mMaxrtt < mLastrtt[i] )
	        mMaxrtt = mLastrtt[i];
	    if( mMinrtt > mLastrtt[i] )
	        mMinrtt = mLastrtt[i];
	  }
	mAvertt += mLastrtt[i];
      }
    mCurlast = mLastrtt[index];
    //pthread_mutex_unlock( &mStats_mutex );

    if( i > 4 )
      {
	// remove the high value and the low value from the average
	mAvertt = mAvertt - mMinrtt - mMaxrtt;
	i -= 2;
      }
    mAvertt = mAvertt / ( static_cast<double>(i) );
}


/* ---------------------------------------------------------
 *  printResults
 *
 * --------------------------------------------------------- */

void
PingResults::printResults(void)
{
    if( mMulticast )
	cout << "\nMulticast RTT (msec) to " << mPeeraddr;
    else
      	cout <<  "\nUnicast RTT (msec) to " << mRemote;

    cout << "\n\tTest       : " << mTestParam;
    cout << "\n\tAverage RTT: " << mAvertt;
    cout << "\n\tMaximum RTT: " << mMaxrtt;
    cout << "\n\tMinimum RTT: " << mMinrtt;
    cout << "\n\tLast    RTT: " << mCurlast << endl;

    return;
}

/* ---------------------------------------------------------
 *  netLogResults_ulm: method for sending results to netlogger
 *
 * --------------------------------------------------------- */

#if USE_NETLOGGER
void
PingResults::netLogResults_ulm(struct _loginfo * Nlh)
{
#ifdef SENDER_LOGS_ONLY
    if ( Nlh != NULL )
    {
       fprintf(stderr,
          "Comparing sender '%s' with host '%s'\n", mSender.c_str(),Nlh->hname);
       if ( !strcmp(mSender.c_str(),Nlh->hname) ) return;
    }
#endif // SENDER_LOGS_ONLY 

    string testArgs(mTestParam);
    replaceSpaces(testArgs);


    string formatTest("ARGS=%s NL.SND=%s NL.RCV=%s AVE.MSEC=%.3f MAX.MSEC=%.3f MIN.MSEC=%.3f LST.MSEC=%.3f");
    if( mMulticast ) {
	printDebug( 3, "Netlog multiping\n");
        if( NetLoggerWrite(Nlh, "pingIPMulti", const_cast<char*>(formatTest.c_str()), 
		    testArgs.c_str(), mSender.c_str(), mReceiver.c_str(), mAvertt, mMaxrtt, mMinrtt, mCurlast ) < 0 )
  	    cerr << "ERROR:NetLoggerWrite  PingResults::multiping\n";
    }
    else {
	printDebug( 3, "Netlog udpping\n");
        if( NetLoggerWrite(Nlh, "pingUDP", const_cast<char*>(formatTest.c_str()), 
		    testArgs.c_str(), mSender.c_str(), mReceiver.c_str(), mAvertt, mMaxrtt, mMinrtt, mCurlast ) < 0 )
  	    cerr << "ERROR:NetLoggerWrite  PingResults::udpping\n";
    }
    return;
}
#endif

/* ---------------------------------------------------------------------------
 * replaceSpaces: Replaces all instances of ' ' with '_', and removes spaces at
 *      head and tail of string.
 *
 * ---------------------------------------------------------------------------*/

void
PingResults::replaceSpaces( string& raw )
{
    char space(' ');
    string::size_type begIdx = raw.find_first_not_of( space );
    string::size_type endIdx = raw.find_last_not_of( space );

    if ( begIdx != string::npos ){
        char us('_');
        for( string::size_type i=begIdx; i<=endIdx; i++ ){
            if( raw[i]==space )
                raw[i] = us;
        }
        raw = raw.substr(begIdx, endIdx-begIdx+1);
    }
    else
        raw = "???";
}
